@@ -10,16 +10,29 @@ from bgesdk.client import API
 from bgesdk.error import APIError
 from bgesdk.management.command import BaseCommand
 from bgesdk.management.constants import (
-    TAB_CHOICES, TITLE_NAME, API_TABLE, DEFAULT_TOKEN_SECTION,
-    DEFAULT_OAUTH2_SECTION, DEFAULT_MODEL_TIMEOUT
+    API_TABLE,
+    DEFAULT_MODEL_TIMEOUT,
+    DEFAULT_OAUTH2_SECTION,
+    DEFAULT_TOKEN_SECTION,
+    TAB_CHOICES,
+    TITLE_NAME,
 )
 from bgesdk.management.utils import (
-    config_get, get_active_project, read_config, get_home,
-    output, SYS_STR
+    config_get,
+    get_active_project,
+    get_home,
+    output,
+    output_json,
+    read_config,
+    SYS_STR
 )
 from bgesdk.management.validate import validator_doc
 
 
+TABLE_MESSAGE = '| {} | {} |'
+TITLE_MESSAGE = '### {}'
+
+
 class Command(BaseCommand):
 
     order = 6
@@ -28,8 +41,17 @@ class Command(BaseCommand):
 
     def add_arguments(self, parser):
         home = get_home()
-        doc_ps = parser.add_subparsers(help='对模型文档进行预览和发布')
-
+        try:
+            doc_ps = parser.add_subparsers(
+                help='对模型文档进行预览和发布'
+            )
+        except TypeError:
+            # required: Whether or not a subcommand must be provided, 
+            # by default False (added in 3.7)
+            doc_ps = parser.add_subparsers(
+                help='对模型文档进行预览和发布',
+                required=False
+            )
         init_p = doc_ps.add_parser(
             'init',
             formatter_class=argparse.ArgumentDefaultsHelpFormatter,
@@ -88,23 +110,27 @@ class Command(BaseCommand):
             home = get_home()
         docs_dir = join(home, name)
         if exists(docs_dir):
-            output('错误！{} 已存在 '.format(docs_dir))
+            output('[red]错误！{} 已存在[/red] '.format(docs_dir))
             sys.exit(1)
         if not exists(home):
-            output('错误！无法找到 home 目录 {}。'.format(home))
+            output('[red]错误！无法找到 home 目录 {}。[/red]'.format(home))
             sys.exit(1)
         with os.popen('docsify init {}'.format(docs_dir)) as f:
             content = f.read()
         if content:
-            output('docsify 项目已初始化，路径为：{}'.format(docs_dir))
-            output('请跳转至项目目录下。')
+            output(
+                '[green]docsify 项目已初始化，路径为：[/green]{}'.format(docs_dir)
+            )
+            output('[green]请跳转至项目目录下。[/green]')
 
     def get_docs_dir(self, home=None):
         if home is None:
             home = get_home()
         doc_path = join(home, 'index.html')
         if not exists(doc_path):
-            output('请确认当前目录或者所输入的项目路径是否为 docsify 项目的根目录。')
+            output(
+                '[red]请确认当前目录或者输入的项目路径是否为 docsify 项目根目录。[/red]'
+            )
             sys.exit(1)
         return home
 
@@ -113,17 +139,17 @@ class Command(BaseCommand):
         with os.popen(command) as f:
             content = f.read()
         if not content:
-            output('请先安装 docsify，参考 https://docsify.js.org/#/quickstart')
+            output('[red]请先安装 docsify，参考 https://docsify.js.org/[/red]')
             sys.exit(1)
         path = args.path
         if not exists(path):
-            output('文件路径：{} 有误，请检查。'.format(path))
+            output('[red]文件路径：{} 有误，请检查。[/red]'.format(path))
             sys.exit(1)
         docs_dir = self.get_docs_dir(home=args.home)
         doc_data = json.load(open(path))
         result = validator_doc(doc_data)
         if result['valid'] is False:
-            output('文件内容有误，错误内容：{}'.format(result['errors']))
+            output('[red]文件内容有误，错误内容：[/red]{}'.format(result['errors']))
             sys.exit(1)
         doc_tab = doc_data['doc_tab']
         model_id = doc_data['model_id']
@@ -136,8 +162,7 @@ class Command(BaseCommand):
             language = content['language']
             doc_name = content['doc_name']
             if language == 'en':
-                file_dir = join(
-                    docs_dir, 'en', 'model_center')
+                file_dir = join(docs_dir, 'en', 'model_center')
                 req_path = 'en/model_center/{}.md'.format(model_id)
             if not exists(file_dir):
                 os.makedirs(file_dir)
@@ -146,7 +171,6 @@ class Command(BaseCommand):
             sidebar = '        * [{}]({})'.format(doc_name, req_path)
             sidebar_lines.append(sidebar)
             self._make_doc(content, file_path, model_id)
-
         self._write_to_sidebar(doc_tab, sidebar_path, sidebar_lines)
         self._write_index(docs_dir)
         os.system('docsify serve {} --port 3000'.format(docs_dir))
@@ -158,64 +182,61 @@ class Command(BaseCommand):
         developer = 'Developer: {}'.format(content.get('developer', ''))
         content_title = '# {}'.format(content.get('content_title', ''))
         ctime = 'Ctime: {}'.format(
-            datetime.strftime(datetime.now(), '%Y-%m-%d %H:%M:%S'))
+            datetime.strftime(datetime.now(), '%Y-%m-%d %H:%M:%S')
+        )
         version_line = 'Version:'
-        join_lines = [content_title, ctime, developer, version_line,
-                      divid_line]
+        join_lines = [
+            content_title,
+            ctime,
+            developer,
+            version_line,
+            divid_line
+        ]
         for key, value in TITLE_NAME.items():
             title_line = '#### {}'.format(value)
             join_model_id = None
             science_detail_title = None
             if key == 'brief_intro':
-                title_line = '### {}'.format(value)
+                title_line = TITLE_MESSAGE.format(value)
                 join_model_id = '**model_id**: `{}`'.format(model_id)
-                science_detail_title = '### {}'.format('科学细节')
+                science_detail_title = TITLE_MESSAGE.format('科学细节')
             handle_content = self._join_content(content.get(key, {}))
             join_lines.append(title_line)
             join_lines.extend(handle_content)
             join_lines.append(join_model_id)
             join_lines.append(science_detail_title)
-        api_title = '### {}'.format('API调用')
+        api_title = TITLE_MESSAGE.format('API调用')
         join_lines.append(api_title)
-
         for key, value in API_TABLE.items():
-
             params_line = '![{}](https://img.shields.io/badge/' \
                           '{}-{}-blue)'.format(key, value, key)
             params = dict()
-            if key == 'QueryParams':
-                pass
-            elif key == 'Success':
+            if key == 'Success':
                 params = content.get('return_params', None)
             elif key == 'State':
                 params = content.get('state_explain', None)
             params_table = self._join_table(params)
             join_lines.append(params_line)
             join_lines.append(params_table)
-
         example_result_line = '![Success](https://img.shields.io/badge/' \
                               '输出-Success-green)'
         json_str = '```json'
-        example_result = content.get('example_result')
-        example_result_json = ''
-        if example_result:
-            example_result_json = json.dumps(
-                example_result, sort_keys=False, indent=4)
+        example_result = content.get('example_result', {})
+        example_result_json = json.dumps(
+            example_result, sort_keys=False, indent=4)
         last_json_str = '```'
         join_lines.append(example_result_line)
         join_lines.append(json_str)
         join_lines.append(example_result_json)
         join_lines.append(last_json_str)
         join_lines.append(divid_line)
-
-        ref_line = '### {}'.format('参考文献')
+        ref_line = TITLE_MESSAGE.format('参考文献')
         join_lines.append(ref_line)
         refs = content.get('ref', [])
         ref_lines = []
-        if refs:
-            for index, ref in enumerate(refs):
-                ref_str = r'\[{}]: {}{}'.format(index + 1, ref, '<br>')
-                ref_lines.append(ref_str)
+        for index, ref in enumerate(refs):
+            ref_str = r'\[{}]: {}{}'.format(index + 1, ref, '<br>')
+            ref_lines.append(ref_str)
         join_lines.append(ref_lines)
         for line in join_lines:
             if isinstance(line, list):
@@ -224,7 +245,6 @@ class Command(BaseCommand):
             elif isinstance(line, str):
                 lines.append(line + line_feed)
             lines.append(line_feed)
-
         with open(file_path, 'w') as f:
             f.writelines(lines)
 
@@ -247,8 +267,8 @@ class Command(BaseCommand):
                             data = params[index]
                         except:
                             output(
-                                'arguments中的关键词数组应大于'
-                                '等于templates中的关键词数组')
+                                '[red]arguments中的关键词数组应大于'
+                                '等于templates中的关键词数组[/red]')
                             sys.exit(1)
                         image_md = '''{}<br><center><img src="{}" 
                         width="386" height="386" /><div style="color: #999;
@@ -263,12 +283,12 @@ class Command(BaseCommand):
 
     def _join_table(self, params):
         table_lines = []
-        table_1 = '| {} | {} |'.format('数据名'.ljust(29), '描述'.ljust(29))
-        table_2 = '| {} | {} |'.format('-' * 31, '-' * 31)
+        table_1 = TABLE_MESSAGE.format('数据名'.ljust(29), '描述'.ljust(29))
+        table_2 = TABLE_MESSAGE.format('-' * 31, '-' * 31)
         table_lines.append(table_1)
         table_lines.append(table_2)
         for key, value in params.items():
-            param_line = '| {} | {} |'.format(
+            param_line = TABLE_MESSAGE.format(
                 str(key).ljust(31), str(value))
             table_lines.append(param_line)
         return table_lines
@@ -287,7 +307,7 @@ class Command(BaseCommand):
     def _write_index(self, docs_dir):
         index_path = join(docs_dir, 'index.html')
         if exists(index_path) is False:
-            output('docs 目录下无 index.html')
+            output('[red]docs 目录下无 index.html[/red]')
             sys.exit(1)
         lines = []
         with open(index_path, 'r') as f:
@@ -319,7 +339,7 @@ class Command(BaseCommand):
         input_value = input('？请输入需要发布的 json 文件路径：')
         if input_value:
             if not exists(input_value):
-                output('文件路径：{} 有误，请检查。'.format(input_value))
+                output('[red]文件路径：{} 有误，请检查。[/red]'.format(input_value))
                 sys.exit(1)
             doc_data = json.load(open(input_value))
             doc_tab = doc_data['doc_tab']
@@ -328,12 +348,12 @@ class Command(BaseCommand):
             try:
                 result = api.upload_model_doc(doc_tab, model_id, doc_content)
             except APIError as e:
-                output('模型文档上传失败：{}'.format(e))
+                output('[red]模型文档上传失败：[/red]')
+                output_json(e.result)
                 sys.exit(1)
             output('模型文档上传结果：')
-            output(json.dumps(result.json(), indent=4, ensure_ascii=False))
-
+            output_json(result)
         else:
-            output('请输入需要预览的文档路径')
+            output('[red]请输入需要预览的文档路径[/red]')
             sys.exit(1)
 
